@@ -67,7 +67,6 @@ pub fn call_record(xs: &CalcitItems) -> Result<Calcit, String> {
   }
 }
 
-/// TODO the name is current `make-record`
 pub fn record_from_map(xs: &CalcitItems) -> Result<Calcit, String> {
   match (xs.get(0), xs.get(1)) {
     (Some(Calcit::Record(name, fields, _values)), Some(Calcit::Map(ys))) => {
@@ -99,16 +98,16 @@ pub fn record_from_map(xs: &CalcitItems) -> Result<Calcit, String> {
       pairs.sort_by(|(a, _), (b, _)| a.cmp(b));
       Ok(Calcit::Record(name.clone(), fields.clone(), values))
     }
-    (Some(a), Some(b)) => Err(format!("make-record expected a record and a map, got {} {}", a, b)),
-    (_, _) => Err(format!("make-record expected 2 arguments, got {:?}", xs)),
+    (Some(a), Some(b)) => Err(format!("&record:from-map expected a record and a map, got {} {}", a, b)),
+    (_, _) => Err(format!("&record:from-map expected 2 arguments, got {:?}", xs)),
   }
 }
 
 pub fn get_record_name(xs: &CalcitItems) -> Result<Calcit, String> {
   match xs.get(0) {
     Some(Calcit::Record(name, ..)) => Ok(Calcit::Str(name.to_owned())),
-    Some(a) => Err(format!("get-record-name expected record, got: {}", a)),
-    None => Err(String::from("get-record-name expected record, got nothing")),
+    Some(a) => Err(format!("&record:get-name expected record, got: {}", a)),
+    None => Err(String::from("&record:get-name expected record, got nothing")),
   }
 }
 pub fn turn_map(xs: &CalcitItems) -> Result<Calcit, String> {
@@ -120,17 +119,17 @@ pub fn turn_map(xs: &CalcitItems) -> Result<Calcit, String> {
       }
       Ok(Calcit::Map(ys))
     }
-    Some(a) => Err(format!("turn-map expected a record, got {}", a)),
-    None => Err(String::from("turn-map expected 1 argument, got nothing")),
+    Some(a) => Err(format!("&record:to-map expected a record, got {}", a)),
+    None => Err(String::from("&record:to-map expected 1 argument, got nothing")),
   }
 }
-pub fn relevant_record_ques(xs: &CalcitItems) -> Result<Calcit, String> {
+pub fn matches(xs: &CalcitItems) -> Result<Calcit, String> {
   match (xs.get(0), xs.get(1)) {
     (Some(Calcit::Record(left, left_fields, ..)), Some(Calcit::Record(right, right_fields, ..))) => {
       Ok(Calcit::Bool(left == right && left_fields == right_fields))
     }
-    (Some(a), Some(b)) => Err(format!("relevant-record? expected 2 records, got {} {}", a, b)),
-    (_, _) => Err(format!("relevant-record? expected 2 arguments, got {:?}", xs)),
+    (Some(a), Some(b)) => Err(format!("&record:matches? expected 2 records, got {} {}", a, b)),
+    (_, _) => Err(format!("&record:matches? expected 2 arguments, got {:?}", xs)),
   }
 }
 
@@ -231,3 +230,64 @@ pub fn assoc(xs: &CalcitItems) -> Result<Calcit, String> {
     (None, ..) => Err(format!("record:assoc expected 3 arguments, got: {:?}", xs)),
   }
 }
+
+pub fn extend_as(xs: &CalcitItems) -> Result<Calcit, String> {
+  if xs.len() != 4 {
+    return Err(format!("record:extend-as expected 4 arguments, got: {:?}", xs));
+  }
+  match (xs.get(0), xs.get(1), xs.get(2), xs.get(3)) {
+    (Some(Calcit::Record(_name, fields, values)), Some(n), Some(a), Some(new_value)) => match a {
+      Calcit::Str(s) | Calcit::Keyword(s) | Calcit::Symbol(s, ..) => match find_in_fields(fields, s) {
+        Some(_pos) => Err(format!("field `{}` already existed", s)),
+        None => {
+          let mut next_fields: Vec<String> = vec![];
+          let mut next_values: Vec<Calcit> = vec![];
+          let mut inserted: bool = false;
+
+          for (i, k) in fields.iter().enumerate() {
+            if inserted {
+              next_fields.push(k.to_owned());
+              next_values.push(values[i].to_owned());
+            } else {
+              match s.cmp(k) {
+                Ordering::Less => {
+                  next_fields.push(s.to_owned());
+                  next_values.push(new_value.to_owned());
+
+                  next_fields.push(k.to_owned());
+                  next_values.push(values[i].to_owned());
+                  inserted = true;
+                }
+                Ordering::Greater => {
+                  next_fields.push(k.to_owned());
+                  next_values.push(values[i].to_owned());
+                }
+                Ordering::Equal => {
+                  unreachable!("does not equal")
+                }
+              }
+            }
+          }
+          if !inserted {
+            next_fields.push(s.to_owned());
+            next_values.push(new_value.to_owned());
+          }
+
+          let new_name: Result<String, String> = match n {
+            Calcit::Str(s) | Calcit::Keyword(s) | Calcit::Symbol(s, ..) => Ok(s.to_owned()),
+            _ => Err(format!("")),
+          };
+
+          Ok(Calcit::Record(
+            new_name?.to_owned(),
+            next_fields.to_owned(),
+            next_values.to_owned(),
+          ))
+        }
+      },
+      a => Err(format!("invalid field `{}` for {:?}", a, fields)),
+    },
+    (Some(a), ..) => Err(format!("record:extend-as expected a record, got: {}", a)),
+    (None, ..) => Err(format!("record:extend-as expected 4 arguments, got: {:?}", xs)),
+  }
+}
